<?php
/**
 * Base class for running multiple tests with the same data.
 *
 * @package components
 * @subpackage password_strength_check
 * @version $Id: password_strength_check_collection.class.php 5593 2012-03-13 12:42:25Z avel $
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 * @author Kostas Koukopoulos <kouk@noc.uoa.gr>
 * @author Alexandros Vellis <avel@noc.uoa.gr>
 * @todo reinstate some OO private/public declarations after complete switch to PHP5... (see r2641)
 */

/**
 * Base class for running multiple tests with the same data.
 *
 * Child classses must define functions that accept some form
 * of test data and return NULL on success or an explanation of
 * the error.
 *
 */
class passwordStrengthCheckCollection
{

    protected $_enabled_tests = array();
    protected $_failed_tests = array();
    protected $_failed_tests_msgs = array();
    protected $_successful_tests = array();

    /**
     * Function that runs all test methods with specified data.
     * @param mixed $data
     * @param mixed $desired_tests  Tests to enable. If true, all tests will be
     *        on. Otherwise it will be an array of the test names to enable.
     *        E.g. array('retypetest', 'lcstest')
     *
     * @return array An array of the tests that failed.
     */
    public function runTests($data, $desired_tests = true) {
        $results = array();

        $this->_enabled_tests = $this->enabledTests($desired_tests);

        foreach($this->_enabled_tests as $test) {
            $tmp = $this->$test($data);
            $results[$test] = $tmp;

            if($tmp) {
                $this->_failed_tests[] = $test;
                $this->_failed_tests_msgs[] = $tmp;
            } else {
                $this->_successful_tests[] = $test;
            }
        }
        return array_filter($results);  // filter out all succesfull tests, report only errors
    }

    /**
     * Which tests are enabled?
     * @return array
     */
    public function enabledTests($desired_tests) {
        foreach(get_class_methods($this) as $test) {
            if (strcasecmp(substr($test,-4), 'test') == 0 &&
             ($desired_tests === true || in_array(strtolower($test),$desired_tests))) {
                $_enabled_tests[] = $test;
            }
        }
        return $_enabled_tests;
    }

    /**
     * Retrieve All Messages (Information messages)
     *
     * @param mixed $desired_tests Tests to get messages for, or boolean true to get
     *    all messages
     * @return array Associative array, key: name of test, value: message
     */
    public function allTestMessages($desired_tests = true) {
        $all_messages = array();
        $_enabled_tests = $this->enabledTests($desired_tests);

        foreach($_enabled_tests as $test) {
            $t = substr($test, 0, -4);
            if(isset($this->messages[$t]) && !empty($this->messages[$t][0])) {
                $all_messages[$test] = $this->messages[$t][0]; 
            }
        }
        return $all_messages;
    }

    /**
     * Getter: enabled tests
     * @return array
     */
    public function retrieveEnabledTests() {
        return $this->_enabled_tests;
    }

    /**
     * Getter: failed tests
     * @return array
     */
    public function retrieveFailedTests() {
        return $this->_failed_tests;
    }

    /**
     * Getter: failed tests with error messages
     * @return array
     */
    public function retrieveFailedTestsMessages() {
        return $this->_failed_tests_msgs;
    }
    
    /**
     * Getter: tests that succeeded
     * @return array
     */
    public function retrieveSuccessfulTests() {
        return $this->_successful_tests;
    }

    /**
     * Getter: number of enabled tests
     * @return int
     */
    public function enabledTestsCount() {
        return count($this->_enabled_tests);
    }

    /**
     * Getter: number of failed tests
     * @return int
     */
    public function failedTestsCount() {
        return count($this->_failed_tests);
    }

    /**
     * Getter: number of successful tests
     * @return int
     */
    public function successfulTestsCount() {
        return count($this->_successful_tests);
    }
}

